import React from "react";
import { Table } from "@ess-ics/ce-ui-common";
import { Grid, Tooltip, Typography, Link } from "@mui/material";
import { IOCStatusIcon } from "../IOCIcons";
import { noWrapText } from "../../common/Helper";

const ownIocsColumns = [
  {
    field: "status",
    headerName: "Status",
    flex: 0,
    headerAlign: "center",
    align: "center"
  },
  {
    field: "namingName",
    headerName: "IOC name",
    width: "15ch",
    sortable: false
  },
  {
    field: "description",
    headerName: "Description",
    width: "20ch",
    sortable: false
  },
  { field: "host", headerName: "Host", width: "10ch", sortable: false },
  { field: "network", headerName: "Network", width: "10ch", sortable: false }
];

const exploreIocsColumns = [
  {
    field: "status",
    headerName: "Status",
    flex: 0,
    headerAlign: "center",
    align: "center"
  },
  {
    field: "namingName",
    headerName: "IOC name",
    width: "15ch",
    sortable: false
  },
  {
    field: "description",
    headerName: "Description",
    width: "20ch",
    sortable: false
  },
  { field: "host", headerName: "Host", width: "10ch", sortable: false },
  { field: "network", headerName: "Network", width: "10ch", sortable: false }
];

const hostDetailsColumns = [
  {
    field: "status",
    headerName: "Status",
    flex: 0,
    headerAlign: "center",
    align: "center"
  },
  {
    field: "namingName",
    headerName: "IOC name",
    width: "8ch",
    sortable: false
  },
  {
    field: "description",
    headerName: "Description",
    width: "20ch",
    sortable: false
  }
];

function createHostLink(host) {
  if (host?.externalHostId) {
    return (
      <Link href={`/hosts/${host?.externalHostId}`}>
        {noWrapText(host.hostName || "---")}
      </Link>
    );
  }

  return "---";
}

function createIocDescription(description) {
  return (
    <>
      {description ? (
        <Tooltip
          title={description}
          arrow
          enterDelay={400}
        >
          <Typography
            style={{
              overflow: "hidden",
              textOverflow: "ellipsis",
              whiteSpace: "nowrap"
            }}
          >
            {description}
          </Typography>
        </Tooltip>
      ) : (
        "---"
      )}
    </>
  );
}

function createTableRowForHostDetails(ioc) {
  const { id, name, namingName, alertSeverity, description } = ioc;

  return {
    id: id,
    status: (
      <Grid
        container
        direction="column"
        justifyContent="center"
        alignItems="center"
      >
        <IOCStatusIcon ioc={ioc} />
      </Grid>
    ),
    namingName: (
      <Link href={`/iocs/${id}`}>{noWrapText(namingName ?? name)}</Link>
    ),
    discrepancy: alertSeverity === "WARNING",
    inconsistentState: alertSeverity === "ERROR",
    description: createIocDescription(description)
  };
}

function createTableRowForOwnIocs(ioc) {
  const { id, activeDeployment, namingName, alertSeverity, description } = ioc;

  return {
    id: id,
    status: (
      <Grid
        container
        direction="column"
        justifyContent="flex-start"
        alignItems="center"
      >
        <IOCStatusIcon ioc={ioc} />
      </Grid>
    ),
    namingName: (
      <Link href={`/iocs/${id}`}>{noWrapText(namingName ?? "---")}</Link>
    ),
    host: createHostLink(activeDeployment?.host),
    network: noWrapText(activeDeployment?.host.network || "---"),
    discrepancy: alertSeverity === "WARNING",
    inconsistentState: alertSeverity === "ERROR",
    iocNotDeployed: !ioc.activeDeployment,
    description: createIocDescription(description)
  };
}

function createTableRowForExploreIocs(ioc) {
  const { id, namingName, createdBy, alertSeverity, host, description } = ioc;

  return {
    id: id,
    status: (
      <Grid
        container
        direction="column"
        justifyContent="center"
        alignItems="center"
      >
        <IOCStatusIcon ioc={ioc} />
      </Grid>
    ),
    namingName: (
      <Link href={`/iocs/${id}`}>{noWrapText(namingName ?? "---")}</Link>
    ),
    host: createHostLink(host),
    network: noWrapText(host?.network || "---"),
    createdBy: noWrapText(createdBy || "---"),
    discrepancy: alertSeverity === "WARNING",
    inconsistentState: alertSeverity === "ERROR",
    iocNotDeployed: !ioc.activeDeployment,
    description: createIocDescription(description)
  };
}

export function IOCTable({
  iocs,
  rowType = "own",
  loading,
  pagination,
  onPage
}) {
  const tableTypeSpecifics = {
    own: [ownIocsColumns, createTableRowForOwnIocs],
    host: [hostDetailsColumns, createTableRowForHostDetails],
    explore: [exploreIocsColumns, createTableRowForExploreIocs]
  };

  const [columns, createRow] = tableTypeSpecifics[rowType];
  const rows = iocs.map((ioc) => createRow(ioc));

  return (
    <Table
      columns={columns}
      rows={rows}
      pagination={pagination}
      onPage={onPage}
      loading={loading}
    />
  );
}
